﻿#region License
/* Copyright 2011 James F. Bellinger <http://www.zer7.com/>

   This software is provided 'as-is', without any express or implied
   warranty. In no event will the authors be held liable for any damages
   arising from the use of this software.
 
   Permission is granted to anyone to use this software for any purpose,
   including commercial applications, and to alter it and redistribute it
   freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you must
   not claim that you wrote the original software. If you use this software
   in a product, an acknowledgement in the product documentation would be
   appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and must
   not be misrepresented as being the original software.

   3. This notice may not be removed or altered from any source
   distribution. 
*/
#endregion

using System;
using System.ComponentModel;
using System.Globalization;

namespace IllusoryStudios.Wpf.LostControls.Converters
{
    public class FitLayoutConverter : TypeConverter
    {
        public override bool CanConvertFrom(ITypeDescriptorContext context, Type sourceType)
        {
            return sourceType == typeof(string);
        }

        public override object ConvertFrom(ITypeDescriptorContext context, CultureInfo culture, object value)
        {
            FitLayout layout = new FitLayout() { LayoutType = FitLayoutType.Auto };

            var str = value as string; double result;
            if (str != null)
            {
                foreach (var part in str.Split(';'))
                {
                    if (part == "Auto")
                    {
                        layout = new FitLayout() { LayoutType = FitLayoutType.Auto };
                    }
                    else if (part == "Shrinkable")
                    { 
                        layout = new FitLayout() { LayoutType = FitLayoutType.Shrinkable };
                    }
                    else if (part.EndsWith("*")
                        && double.TryParse(part.Substring(0, part.Length - 1), NumberStyles.Float, culture, out result)
                        && result > 0)
                    {
                        layout = new FitLayout() { LayoutType = FitLayoutType.Star, Value = result };
                    }
                    else if (part == "*")
                    {
                        layout = new FitLayout() { LayoutType = FitLayoutType.Star, Value = 1 };
                    }
                    else if (double.TryParse(part, NumberStyles.Float, culture, out result) && result > 0)
                    {
                        layout = new FitLayout() { LayoutType = FitLayoutType.Pixel, Value = result };
                    }
                    else if (part.StartsWith("Minimum=")
                        && double.TryParse(part.Substring("Minimum=".Length), NumberStyles.Float, culture, out result)
                        && result >= 0)
                    {
                        layout.Minimum = result;
                    }
                    else
                    {
                        goto fail;
                    }
                }

                return layout;
            }

fail:
            return base.ConvertFrom(context, culture, value);
        }
    }
}