﻿#region License
/* Copyright 2010 James F. Bellinger <http://www.zer7.com/>

   This software is provided 'as-is', without any express or implied
   warranty. In no event will the authors be held liable for any damages
   arising from the use of this software.
 
   Permission is granted to anyone to use this software for any purpose,
   including commercial applications, and to alter it and redistribute it
   freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you must
   not claim that you wrote the original software. If you use this software
   in a product, an acknowledgement in the product documentation would be
   appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and must
   not be misrepresented as being the original software.

   3. This notice may not be removed or altered from any source
   distribution. 
*/
#endregion

using System;
using System.Globalization;
using System.Windows;
using System.Windows.Data;

namespace IllusoryStudios.Wpf.LostControls.Converters
{
    public class VisibilityConverter : IValueConverter, IMultiValueConverter
    {
        public static VisibilityConverter VisibleWhenAllMatch
        {
            get { return new VisibilityConverter() { CompareWith = VisibilityComparisonTarget.Others }; }
        }

        public VisibilityConverter()
        {
            CompareCondition = 0; CompareValue = true;
            CompareWith = VisibilityComparisonTarget.Static;
        }

        public int CompareCondition
        {
            get;
            set;
        }

        public object CompareValue
        {
            get;
            set;
        }

        public VisibilityComparisonTarget CompareWith
        {
            get;
            set;
        }

        public bool Not
        {
            get;
            set;
        }

        bool Compare(object value, object with)
        {
            bool result = false;
            if (value != null)
            {
                var v = value as IComparable;
                if (v != null)
                {
                    try { result = v.CompareTo(with) == CompareCondition; }
                    catch { }
                }
                else if (CompareCondition == 0)
                {
                    try { result = object.Equals(value, with); }
                    catch { }
                }
            }
            return result;
        }

        Visibility Result(bool value)
        {
            return (value ^ Not) ? Visibility.Visible : Visibility.Collapsed;
        }

        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            return Convert(new object[] { value }, targetType, parameter, culture);
        }

        public object Convert(object[] values, Type targetType, object parameter, CultureInfo culture)
        {
            bool result = true;

            switch (CompareWith)
            {
                case VisibilityComparisonTarget.Static:
                    for (int i = 0; i < values.Length; i++) { result &= Compare(values[i], CompareValue); }
                    break;

                case VisibilityComparisonTarget.Parameter:
                    for (int i = 0; i < values.Length; i++) { result &= Compare(values[i], parameter); }
                    break;

                case VisibilityComparisonTarget.Others:
                    for (int i = 0; i < values.Length - 1; i++) { result &= Compare(values[i], values[i + 1]); }
                    break;
            }

            return Result(result);
        }

        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            throw new NotSupportedException();
        }

        public object[] ConvertBack(object value, Type[] targetTypes, object parameter, CultureInfo culture)
        {
            throw new NotSupportedException();
        }
    }
}
