﻿#region License
/* Copyright 2009-2010 James F. Bellinger <http://www.zer7.com/>

   This software is provided 'as-is', without any express or implied
   warranty. In no event will the authors be held liable for any damages
   arising from the use of this software.
 
   Permission is granted to anyone to use this software for any purpose,
   including commercial applications, and to alter it and redistribute it
   freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you must
   not claim that you wrote the original software. If you use this software
   in a product, an acknowledgement in the product documentation would be
   appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and must
   not be misrepresented as being the original software.

   3. This notice may not be removed or altered from any source
   distribution. 
*/
#endregion

using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Markup;

namespace IllusoryStudios.Wpf.LostControls
{
    [ContentProperty("Text")]
    public partial class EditableLabel : UserControl
    {
        public static readonly DependencyProperty AcceptsReturnProperty = TextBox.AcceptsReturnProperty.AddOwner(typeof(EditableLabel));

        public static readonly DependencyProperty IsEditingProperty = DependencyProperty.Register("IsEditing", typeof(bool), typeof(EditableLabel),
            new PropertyMetadata(HandleIsEditingChanged));

        public static readonly DependencyProperty TextProperty = DependencyProperty.Register("Text", typeof(string), typeof(EditableLabel),
            new FrameworkPropertyMetadata(null, FrameworkPropertyMetadataOptions.BindsTwoWayByDefault));

        public static readonly DependencyProperty TextWhenEmptyProperty = DependencyProperty.Register("TextWhenEmpty", typeof(string), typeof(EditableLabel));

        public static readonly DependencyProperty TextWrappingProperty = TextBox.TextWrappingProperty.AddOwner(typeof(EditableLabel));

        public EditableLabel()
        {
            InitializeComponent();
        }

        private void Control_PreviewMouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            if (e.ClickCount == 2)
            {
                e.Handled = true;
                
                IsEditing = !IsEditing;
                if (IsEditing) { TextBox.Focus(); }
            }
        }

        private void Control_IsKeyboardFocusWithinChanged(object sender, DependencyPropertyChangedEventArgs e)
        {
            if (!(bool)e.NewValue) { IsEditing = false; }
        }

        public bool AcceptsReturn
        {
            get { return (bool)GetValue(AcceptsReturnProperty); }
            set { SetValue(AcceptsReturnProperty, value); }
        }

        public bool IsEditing
        {
            get { return (bool)GetValue(IsEditingProperty); }
            set { SetValue(IsEditingProperty, value); }
        }

        public string Text
        {
            get { return (string)GetValue(TextProperty); }
            set { SetValue(TextProperty, value); }
        }

        public string TextWhenEmpty
        {
            get { return (string)GetValue(TextWhenEmptyProperty); }
            set { SetValue(TextWhenEmptyProperty, value); }
        }

        public TextWrapping TextWrapping
        {
            get { return (TextWrapping)GetValue(TextWrappingProperty); }
            set { SetValue(TextWrappingProperty, value); }
        }

        static void HandleIsEditingChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            if (!(bool)e.NewValue && (bool)e.OldValue)
            {
                ((EditableLabel)d).TextBox.GetBindingExpression(TextBox.TextProperty).UpdateSource();
            }
        }

        private void TextBox_KeyDown(object sender, KeyEventArgs e)
        {
            if ((!AcceptsReturn && (e.Key == Key.Enter || e.Key == Key.Return)))
            {
                e.Handled = true; IsEditing = false;
            }
        }
    }
}
