﻿#region License
/* Copyright 2010, 2013 James F. Bellinger <http://www.zer7.com/>

   This software is provided 'as-is', without any express or implied
   warranty. In no event will the authors be held liable for any damages
   arising from the use of this software.
 
   Permission is granted to anyone to use this software for any purpose,
   including commercial applications, and to alter it and redistribute it
   freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you must
   not claim that you wrote the original software. If you use this software
   in a product, an acknowledgement in the product documentation would be
   appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and must
   not be misrepresented as being the original software.

   3. This notice may not be removed or altered from any source
   distribution. 
*/
#endregion

using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Windows;

namespace IllusoryStudios.Wpf.LostControls
{
    public class InputBox : INotifyPropertyChanged
    {
        string _message, _title;

        public event PropertyChangedEventHandler PropertyChanged;

        public InputBox()
        {
            Fields = new ObservableCollection<InputBoxField>();
        }

        public InputBox(string message, string title)
            : this(message, title, Enumerable.Empty<InputBoxField>())
        {

        }

        public InputBox(string message, string title, params string[] fieldNames)
            : this(message, title, fieldNames.Select
            (x => (InputBoxField)new InputBoxTextField(x)))
        {

        }

        public InputBox(string message, string title, IEnumerable<KeyValuePair<string, string>> fields)
            : this(message, title, fields.Select
            (x => (InputBoxField)new InputBoxTextField(x.Key) { Value = x.Value }))
        {

        }

        public InputBox(string message, string title, params InputBoxField[] fields)
            : this(message, title, (IEnumerable<InputBoxField>)fields)
        {

        }

        public InputBox(string message, string title, IEnumerable<InputBoxField> fields)
            : this()
        {
            Message = message; Title = title;
            foreach (var field in fields) { Fields.Add(field); }
        }

        public bool Show()
        {
            return Show(null);
        }

        public bool Show(Window owner)
        {
            for (int i = 0; i < Fields.Count; i++) { Fields[i].Revalidate(); }
            var window = new InputBoxWindow() { DataContext = this, Owner = owner };
            return window.ShowDialog() == true;
        }

        public static string Show(string message, string title)
        {
            return Show(null, message, title, null);
        }

        public static string Show(string message, string title, string defaultValue)
        {
            return Show(null, message, title, defaultValue);
        }

        public static string Show(Window owner, string message, string title)
        {
            return Show(owner, message, title, null);
        }

        public static string Show(Window owner, string message, string title, string defaultValue)
        {
            var field = new InputBoxTextField("", defaultValue ?? "");
            var inputBox = new InputBox(message, title, field);
            return inputBox.Show(owner) ? field.Value : null;
        }

        protected virtual void OnPropertyChanged(string propertyName)
        {
            RaisePropertyChanged(propertyName);
        }

        protected void RaisePropertyChanged(string propertyName)
        {
            var pc = PropertyChanged;
            if (pc != null) { pc(this, new PropertyChangedEventArgs(propertyName)); }
        }

        public ObservableCollection<InputBoxField> Fields
        {
            get;
            private set;
        }

        public string Message
        {
            get { return _message ?? ""; }
            set { _message = value; OnPropertyChanged("Message"); }
        }

        public string Title
        {
            get { return _title ?? ""; }
            set { _title = value; OnPropertyChanged("Title"); }
        }
    }
}
