﻿#region License
/* Copyright 2011 James F. Bellinger <http://www.zer7.com/>

   This software is provided 'as-is', without any express or implied
   warranty. In no event will the authors be held liable for any damages
   arising from the use of this software.
 
   Permission is granted to anyone to use this software for any purpose,
   including commercial applications, and to alter it and redistribute it
   freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you must
   not claim that you wrote the original software. If you use this software
   in a product, an acknowledgement in the product documentation would be
   appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and must
   not be misrepresented as being the original software.

   3. This notice may not be removed or altered from any source
   distribution. 
*/
#endregion

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;

namespace IllusoryStudios.Wpf.LostControls
{
    public class InputBoxComboField : InputBoxField
    {
        int _index;

        public InputBoxComboField()
            : this(null, 0, null)
        {

        }

        public InputBoxComboField(string name)
            : this(name, 0, null)
        {

        }

        public InputBoxComboField(string name, int index, string[] options)
        {
            Options = new ObservableCollection<string>();
            Name = name;

            if (options != null)
            {
                foreach (var option in options) { Options.Add(option ?? ""); }
            }

            Options.CollectionChanged += (sender, e) => Changed();
            Index = index;
        }

        void Changed()
        {
            OnPropertyChanged("Index");
            OnPropertyChanged("Value");
            Revalidate();
        }

        protected override bool Validate()
        {
            return base.Validate() && (Index != -1 || Options.Count == 0);
        }

        public ObservableCollection<string> Options
        {
            get;
            private set;
        }

        public int Index
        {
            get { return _index >= 0 && _index < Options.Count ? _index : -1; }
            set { _index = value; Changed(); }
        }

        public override string Value
        {
            get { return Index >= 0 ? (Options[Index] ?? "") : ""; }
            set { Index = Options.IndexOf(value ?? ""); }
        }
    }
}
