﻿#region License
/* Copyright 2010 James F. Bellinger <http://www.zer7.com/>

   This software is provided 'as-is', without any express or implied
   warranty. In no event will the authors be held liable for any damages
   arising from the use of this software.
 
   Permission is granted to anyone to use this software for any purpose,
   including commercial applications, and to alter it and redistribute it
   freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you must
   not claim that you wrote the original software. If you use this software
   in a product, an acknowledgement in the product documentation would be
   appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and must
   not be misrepresented as being the original software.

   3. This notice may not be removed or altered from any source
   distribution. 
*/
#endregion

using System;
using System.ComponentModel;

namespace IllusoryStudios.Wpf.LostControls
{
    public abstract class InputBoxField : INotifyPropertyChanged
    {
        string _name; bool _isValid;

        public event PropertyChangedEventHandler PropertyChanged;

        public void Revalidate()
        {
            IsValid = Validate();
        }

        protected virtual bool Validate()
        {
            return true;
        }

        protected virtual void OnPropertyChanged(string propertyName)
        {
            RaisePropertyChanged(propertyName);
        }

        protected void RaisePropertyChanged(string propertyName)
        {
            var pc = PropertyChanged;
            if (pc != null) { pc(this, new PropertyChangedEventArgs(propertyName)); }
        }

        public string Name
        {
            get { return _name ?? ""; }
            set { _name = value; OnPropertyChanged("Name"); }
        }

        public abstract string Value
        {
            get;
            set;
        }

        public bool IsValid
        {
            get { return _isValid; }
            private set { _isValid = value; OnPropertyChanged("IsValid"); }
        }
    }
}
