﻿#region License
/* Copyright 2011 James F. Bellinger <http://www.zer7.com/>

   This software is provided 'as-is', without any express or implied
   warranty. In no event will the authors be held liable for any damages
   arising from the use of this software.
 
   Permission is granted to anyone to use this software for any purpose,
   including commercial applications, and to alter it and redistribute it
   freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you must
   not claim that you wrote the original software. If you use this software
   in a product, an acknowledgement in the product documentation would be
   appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and must
   not be misrepresented as being the original software.

   3. This notice may not be removed or altered from any source
   distribution. 
*/
#endregion

using System;
using System.ComponentModel;

namespace IllusoryStudios.Wpf.LostControls
{
    public class InputBoxTextField : InputBoxField
    {
        int _minLength = 0, _maxLength = int.MaxValue;
        string _value; Func<string, bool> _validateCallback;

        public InputBoxTextField()
            : this(null, null)
        {

        }

        public InputBoxTextField(string name)
            : this(name, null)
        {
            
        }

        public InputBoxTextField(string name, string value)
        {
            Name = name; Value = value;
        }

        protected override bool Validate()
        {
            return base.Validate()
                && (ValidateCallback == null || ValidateCallback(Value))
                && Value.Length >= MinLength && Value.Length <= MaxLength;
        }

        public int MaxLength
        {
            get { return _maxLength; }
            set { _maxLength = Math.Max(value, 0); OnPropertyChanged("MaxLength"); Revalidate(); }
        }

        public int MinLength
        {
            get { return _minLength; }
            set { _minLength = Math.Max(value, 0); OnPropertyChanged("MinLength"); Revalidate(); }
        }

        public override string Value
        {
            get { return _value ?? ""; }
            set { _value = value ?? ""; OnPropertyChanged("Value"); Revalidate(); }
        }

        public Func<string, bool> ValidateCallback
        {
            get { return _validateCallback; }
            set { _validateCallback = value; OnPropertyChanged("ValidateCallback"); Revalidate(); }
        }
    }
}
