﻿#region License
/* Copyright 2010 James F. Bellinger <http://www.zer7.com/>

   This software is provided 'as-is', without any express or implied
   warranty. In no event will the authors be held liable for any damages
   arising from the use of this software.
 
   Permission is granted to anyone to use this software for any purpose,
   including commercial applications, and to alter it and redistribute it
   freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you must
   not claim that you wrote the original software. If you use this software
   in a product, an acknowledgement in the product documentation would be
   appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and must
   not be misrepresented as being the original software.

   3. This notice may not be removed or altered from any source
   distribution. 
*/
#endregion

using System;
using System.Collections.Generic;
using System.Globalization;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Input;

namespace IllusoryStudios.Wpf.LostControls
{
    partial class InputBoxWindow : Window
    {
        static readonly RoutedCommand OKCommand = new RoutedCommand();

        InputBox InputBox
        {
            get { return (InputBox)DataContext; }
        }

        public InputBoxWindow()
        {
            Resources.Add("CIN", new CollapseIfNullConverter());

            InitializeComponent();
            OKButton.Command = OKCommand;
            CommandBindings.Add(new CommandBinding(OKCommand, OK_Executed, OK_CanExecute));
        }

        private void OK_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            e.Handled = true;
            DialogResult = true; Close();
        }

        private void OK_CanExecute(object sender, CanExecuteRoutedEventArgs e)
        {
            e.Handled = true;
            foreach (var item in Fields.Items)
            {
                if (!((InputBoxField)item).IsValid) { e.CanExecute = false; return; }
            }
            e.CanExecute = true;
        }

        private void Field_Loaded(object sender, RoutedEventArgs e)
        {
            var fe = (FrameworkElement)sender;
            var field = (InputBoxField)fe.DataContext;
            //var tb = fe.FindName("FieldValue") as Control;
            if (InputBox.Fields.Count >= 1 && InputBox.Fields[0] == field)
                { fe.Focus(); }
        }
    }

    class CollapseIfNullConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            return string.IsNullOrEmpty(value as string)
                ? Visibility.Collapsed : Visibility.Visible;
        }

        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            throw new NotSupportedException();
        }
    }
}
