﻿#region License
/* Copyright 2010 James F. Bellinger <http://www.zer7.com/>

   This software is provided 'as-is', without any express or implied
   warranty. In no event will the authors be held liable for any damages
   arising from the use of this software.
 
   Permission is granted to anyone to use this software for any purpose,
   including commercial applications, and to alter it and redistribute it
   freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you must
   not claim that you wrote the original software. If you use this software
   in a product, an acknowledgement in the product documentation would be
   appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and must
   not be misrepresented as being the original software.

   3. This notice may not be removed or altered from any source
   distribution. 
*/
#endregion

using System;
using System.Reflection;
using Microsoft.Win32;

namespace IllusoryStudios.Wpf.LostControls.Win32
{
    public static class FileAssociationHelper
    {
        static void CheckNull<T>(string name, T value)
        {
            if (value == null) { throw new ArgumentNullException("name"); }
        }

        public static void RegisterAssociation(string association, string handler)
        {
            CheckNull("association", association); CheckNull("handler", handler);
            Registry.SetValue(@"HKEY_CURRENT_USER\Software\Classes\" + association, null, handler, RegistryValueKind.String);
        }

        public static void RegisterHandler(string handler, string description, string quotedCommandLine)
        {
            CheckNull("handler", handler); CheckNull("description", description); CheckNull("quotedCommandLine", quotedCommandLine);
            Registry.SetValue(@"HKEY_CURRENT_USER\Software\Classes\" + handler, null, description, RegistryValueKind.String);
            Registry.SetValue(@"HKEY_CURRENT_USER\Software\Classes\" + handler + @"\shell\open\command", null, quotedCommandLine, RegistryValueKind.ExpandString);
        }

        public static void RegisterHandlerForAssembly(string handler, string description, Assembly assembly)
        {
            CheckNull("assembly", assembly);
            RegisterHandlerForPath(handler, description, assembly.Location);
        }

        public static void RegisterHandlerForPath(string handler, string description, string path)
        {
            CheckNull("handler", handler); CheckNull("description", description); CheckNull("path", path);
            RegisterHandler(handler, description, string.Format("{0} \"%1\"",
                QuoteStringElement(path)));
        }

        public static string QuoteStringElement(string s)
        {
            CheckNull("s", s);
            return "\"" + s.Replace("\"", "\"\"").Replace("%", "%%") + "\"";
        }
    }
}
