﻿#region License
/* Copyright 2010 James F. Bellinger <http://www.zer7.com/>

   This software is provided 'as-is', without any express or implied
   warranty. In no event will the authors be held liable for any damages
   arising from the use of this software.
 
   Permission is granted to anyone to use this software for any purpose,
   including commercial applications, and to alter it and redistribute it
   freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you must
   not claim that you wrote the original software. If you use this software
   in a product, an acknowledgement in the product documentation would be
   appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and must
   not be misrepresented as being the original software.

   3. This notice may not be removed or altered from any source
   distribution. 
*/
#endregion

using System;
using System.Windows;
using System.Windows.Data;

namespace IllusoryStudios.Wpf.LostControls
{
    public static class WindowMemory
    {
        public static readonly DependencyProperty PrefixProperty = DependencyProperty.RegisterAttached
            ("Prefix", typeof(string), typeof(WindowMemory), new PropertyMetadata(HandlePrefixChanged));

        public static readonly DependencyProperty SourceProperty = DependencyProperty.RegisterAttached
            ("Source", typeof(object), typeof(WindowMemory), new PropertyMetadata(HandleSourceChanged));

        public static string GetPrefix(Window window)
        {
            return (string)window.GetValue(PrefixProperty);
        }

        public static void SetPrefix(Window window, string value)
        {
            window.SetValue(PrefixProperty, value);
        }

        public static object GetSource(Window window)
        {
            return window.GetValue(SourceProperty);
        }

        public static void SetSource(Window window, object value)
        {
            window.SetValue(SourceProperty, value);
        }

        static void HandlePrefixChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            var window = d as Window; if (window == null) { return; }
            UpdateBinding(window, GetSource(window), (string)e.NewValue);
        }

        static void HandleSourceChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            var window = d as Window; if (window == null) { return; }
            UpdateBinding(window, e.NewValue, GetPrefix(window));
        }

        static void Bind(Window window, object source, string prefix,
            DependencyProperty property, string name)
        {
            if (source == null || prefix == null)
            {
                BindingOperations.ClearBinding(window, property);
            }
            else
            {
                BindingOperations.SetBinding(window, property, new Binding()
                    {
                        Source = source, Mode = BindingMode.TwoWay,
                        Path = new PropertyPath(prefix + name)
                    });
            }
        }

        static void UpdateBinding(Window window, object source, string prefix)
        {
            Bind(window, source, prefix, Window.WindowStateProperty, "WindowState");
            Bind(window, source, prefix, Window.HeightProperty, "Height");
            Bind(window, source, prefix, Window.WidthProperty, "Width");
            Bind(window, source, prefix, Window.LeftProperty, "Left");
            Bind(window, source, prefix, Window.TopProperty, "Top");
        }
    }
}